
#define REV "ScanTesla  V-TRSSTC-7.40  June 23, 2005  Terry Fritz "

/* This program has no copyright, GPL, copyleft, trademarks, or real owner
   of this Public Domain base version. Anyone can copy, steal, plagiarize,
   change, and use it as they wish.

   The latest version is probably at http://drsstc.com/~terrell/modeling/

   The general guide to the program is at http://www.drsstc.com/~terrell/modeling/ScanTesla.pdf

   This program iterates through and tests a large number of Tesla coil
   parameters in search of those parameters that give the best output sparks
HISTORY
	V-1.00 May 14, 2005	Terry   Original starting version
	V-2.00 June 10, 2005 Terry  Added Antonio's Calculation Engine "ACE"
	V-3.00 June 11, 2005 Terry  Cleaned up everything and did a lot of testing
	V-4.00 June 12, 2005 Terry	Added automatic dt and conventional coil hooks
	V-5.00 June 12, 2005 Terry  Added file input
	V-5.10 June 13, 2005 Terry  Changed all %e, %f, %d to %le, %lf, %ld.  Removed <conio.h>.
	V-5.20 June 13, 2005 Terry	Added revision printout, fixed peak time bug, added Cprimary RMS current.
	V-6.00 June 14, 2005 Terry  Added output file of last best waveform's state data. Fixed T1_inc"1" bug.
								Added Bang Energy Limit.

	V-6.10 June 14, 2005  Terry  Change goal time printout. Fixed Cpri RMS current.
	V-6.11 June 16, 2005  Terry  Took out t variable and fixed comment line.  Fixed T = 1uS bug in putput file.
	V-6.20 June 16, 2005  Terry  Added streamer power profile.
	V-6.21 June 17, 2005  Terry  Added load energy rise time.  Cleaned up messy code.
	V-6.22 June 18, 2005  Terry  Fixed Vin/Vin_old problem.  Few comment fixes.
	V-6.23 June 19, 2005  Terry  In DRSSTC case, Vin = -Vin after dwell time.
	V-6.24 June 19, 2005  Terry  Added antiparallel diode effects for DRSSCT after dwell time shutdown.
	V-7.00 June 20, 2005  Terry	 Added full data output file.  Added Vin to model output file.
	V-7.01 June 21, 2005  Terry  Fixed Vin in wavevorms.txt file header.
	V-7.10 June 23, 2005  Terry  Added Dwell Time loop
	V-7.20 June 23, 2005  Terry  Automatic L1 Range
	V-7.30 June 23, 2005  Terry  Added BPS style calculations Fixed fabs vs abs bug <- BIG BUG!!   Helped screen Jitter
	V-TRSSTC-7.30 June 23, 2005  Antonio/Terry  Adding in Antonio's new TRSSTC engine!!
	V-TRSSTC-7.40 June 27, 2005  Antonio/Terry  Adding in Antonio's new TRSSTC engine with K23 and K13 coupling




FUTURE IMPROVMENTS

Marco suggests using random parameters or genetic programming to scan large areas.


NOTES

	Antonio supplies the magic calculating engine for solving the nodes.
	I basically just add the input and output scanning stuff.


/*
TRSSTC simulator
By Antonio Carlos M. de Queiroz acmq@coe.ufrj.br
Version 1.0 - 26/6/2005
*/

/*
This program simulates the structure:
    +v1-                  -i3+
 +---C1--R1--+   +--R2--+--L3--R3--+--R4--+
+|         + |   | +    |  +       |  +   |  +
vin       i1 L1 L2 i2   C2 v2      C3 v3  C4 v4
-|         - |   | -    |  -       |  -   |  -
 +-----------+   +------+----------+------+
The three coils are coupled, with coupling coefficients k12, k13, and k23.
*/
/* To reduce to a DRSSTC:
Split L2 into L2 and L3
Multiply k12 by sqrt((L2+L3)/L2) and make k13=k23=0
Split R2 into R2 and R3
Make C2 small
*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
//#include <conio.h>



/* INPUTS */


	double C1 = 0.0;  				/* Cprimary in Farads */
	double  C1_start = 0.0;
	double  C1_stop = 0.0;
	double  C1_inc = 0.0;
	double R1 = 0.0;				/* Rprimary in Ohms */
	double	R1_start = 0.0;
	double	R1_stop = 0.0;
	double	R1_inc = 0.0;
	double L1 = 0.0;				/* Lprimary in Heneries */
	double	L1_start = 0.0;
	double	L1_stop = 0.0;
	double	L1_inc = 0.0;
	double L2 = 0.0;				/* Lsecondary in Heneries */
	double	L2_start = 0.0;
	double	L2_stop = 0.0;
	double	L2_inc = 0.0;
	double K12 = 0.0;				/* primary to secondary coupling coefficient */
	double	K12_start = 0.0;
	double  K12_stop = 0.0;
	double	K12_inc = 0.0;
	double K23 = 0.0;				/* primary to secondary coupling coefficient */
	double	K23_start = 0.0;
	double  K23_stop = 0.0;
	double	K23_inc = 0.0;
	double K13 = 0.0;				/* primary to secondary coupling coefficient */
	double	K13_start = 0.0;
	double  K13_stop = 0.0;
	double	K13_inc = 0.0;
	double R2 = 0.0;				/* Rsecondary in Ohms */
	double 	R2_start = 0.0;
	double 	R2_stop = 0.0;
	double	R2_inc = 0.0;
	double C2 = 0.0;				/* Cterminal in Farads */
	double	C2_start = 0.0;
	double 	C2_stop = 0.0;
	double 	C2_inc = 0.0;
	double L3 = 0.0;				/* Ltri in Herneries */
	double  L3_start = 0.0;
	double  L3_stop = 0.0;
	double  L3_inc = 0.0;
	double R3 = 0.0;				/* Rtri in Ohms */
	double	R3_start = 0.0;
	double 	R3_stop = 0.0;
	double 	R3_inc = 0.0;
	double C3 = 0.0;				/* Ctri in Farads */
	double	C3_start = 0.0;
	double	C3_stop = 0.0;
	double	C3_inc = 0.0;
	double C4 = 0.0;				/* Cload in Ohms */
	double	C4_start = 0.0;
	double 	C4_stop = 0.0;
	double 	C4_inc = 0.0;
	double R4 = 0.0;				/* Rload in Ohms */
	double	R4_start = 0.0;
	double 	R4_stop = 0.0;
	double 	R4_inc = 0.0;
	double T1 = 0.0;				/* DRSSTC T1 time in Seconds */
	double	T1_start = 0.0;
	double	T1_stop = 0.0;
	double	T1_inc = 0.0;			/* Negative value for automatic dt */
	double DwellTime = 0.0;			/* T1 on time */
	double 	DwellTime_start = 0.0;
	double	DwellTime_stop = 0.0;
	double	DwellTime_inc = 0.0;

	double Vrail = 0.0;				/* DRSSTC input voltage in Volts */
	double VCpri_init = 0.0;		/* Initital voltage in Cp for conventional coil case */
	double Current_Limit = 0.0; 	/* Primary Current Limit (set high for conventional coil) */
	double CoilPower_Limit = 0.0;
	int Goal_Type = 0;				/* 0 = maximum VCtop voltage  1 = maximum steamer power */
	long int PrintNumber = 0.0;



// Internal Variables

	double Vin = 0.0;
	double VCpri_max = 0.0;
	double ILpri_max = 0.0;
	double VCsec_max = 0.0;
	double VCtri_max = 0.0;
	double Eload = 0.0;				/* Power to R3/bang */
	double CoilPower = 0.0;  		/* Bang power (watts/bang) */
	double Cpri_RMSi = 0.0;
	long int GoalNum = 0.0;
	double T1_high = 0.0;
	long int ModelNumber = 0;
	double goal = 0.0;
	double goal_max = 0.0;
	int fail = 0;
	double pi = 3.14159265359;
	double T1_inc1 = 0.0;
	double L1_inc1 = 0.0;
	double BPS = 0.0;
	char buffer[256];
	long int LoopTotal = 1;
	int screen = 0;

	double Eload0 = 0.0;			/*variables for streamer rise time*/
	double Eload1 = 0.0;
	double TEload = 0.0;
	double TEload0 = 0.0;
	double TEload1 = 0.0;

	double Vin_old = 0.0;


#define filename1 "output.txt"
#define filename2 "input.txt"
#define filename3 "waveforms.csv"
#define filename4 "outputdata.csv"

typedef double matrix [8][8];
typedef double vector [8];
int i,j,k;
matrix A,MT,G;
vector B,X0,X1,Xn;
FILE* output;
FILE* input;
FILE* option;
FILE* data;




/*Routine that inverts a matrix in place */
void Invert(int n, matrix M)
{
  int i,k,j;
  double d,e;

  for (k=1; k<=n; k++) {
    d=M[k][k]; // May need pivotal condensation, omitted
    if (fabs(d)<1e-12) {
      printf("Something wrong (line %d). Inversion impossible\n",k);
      exit(0);
    }
    M[k][k]=1/d;
    for (j=1; j<=n; j++)
      if (j!=k) M[k][j]=-M[k][j]/d;
    for (i=1; i<=n; i++) {
      if (i!=k) {
        e=M[i][k];
        M[i][k]=M[i][k]/d;
        for (j=1; j<=n; j++)
          if (j!=k) M[i][j]=M[i][j]+M[k][j]*e;
      }
    }
  }
}




int main(void)
{

/* File input of above parameters */

input=fopen(filename2,"r");

while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C1_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C1_stop);
        	break;
        	}
	}




while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C1_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R1_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R1_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R1_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L1_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L1_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L1_inc1);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L2_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L2_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L2_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K12_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K12_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K12_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K23_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K23_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K23_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K13_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K13_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&K13_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R2_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R2_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R2_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C2_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C2_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C2_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L3_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L3_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&L3_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R3_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R3_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R3_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C3_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C3_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C3_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C4_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C4_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&C4_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R4_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R4_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&R4_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&T1_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&T1_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&T1_inc1);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&Vrail);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&VCpri_init);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&BPS);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&DwellTime_start);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&DwellTime_stop);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&DwellTime_inc);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&Current_Limit);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%le",&CoilPower_Limit);
        	break;
        	}
	}


while (1)
	{
	buffer[0]= (char)0;
	fgets(buffer,256,input);
	if ( buffer[0] != '*')
        	{
        	sscanf(buffer,"%i",&Goal_Type);
        	break;
        	}
	}




// Print out the input variables to screen and a file.

fclose(input);

printf(REV "\n");
printf("C1 %le %le %le \n", C1_start, C1_stop, C1_inc);
printf("R1 %le %le %le \n", R1_start, R1_stop, R1_inc);
printf("L1 %le %le %le \n", L1_start, L1_stop, L1_inc1);
printf("L2 %le %le %le \n", L2_start, L2_stop, L2_inc);
printf("K12 %le %le %le \n", K12_start, K12_stop, K12_inc);
printf("K23 %le %le %le \n", K23_start, K23_stop, K23_inc);
printf("K13 %le %le %le \n", K13_start, K13_stop, K13_inc);
printf("R2 %le %le %le \n", R2_start, R2_stop, R2_inc);
printf("C2 %le %le %le \n", C2_start, C2_stop, C2_inc);
printf("L3 %le %le %le \n", L3_start, L3_stop, L3_inc);
printf("R3 %le %le %le \n", R3_start, R3_stop, R3_inc);
printf("C3 %le %le %le \n", C3_start, C3_stop, C3_inc);
printf("C4 %le %le %le \n", C4_start, C4_stop, C4_inc);
printf("R4 %le %le %le \n", R4_start, R4_stop, R4_inc);
printf("T1 %le %le %le \n", T1_start, T1_stop, T1_inc1);
printf("Vrail %le \n", Vrail);
printf("VCpri_init %le \n", VCpri_init);
printf("BPS %le \n", BPS);
printf("DwellTime %le %le %le \n", DwellTime_start, DwellTime_stop, DwellTime_inc);
printf("Current_Limit %le \n", Current_Limit);
printf("CoilPower_Limit %le \n", CoilPower_Limit);
printf("Goal Type %i \n", Goal_Type);

output=fopen(filename1,"w");

fprintf(output, REV "\n");
fprintf(output,"C1 %le %le %le \n", C1_start, C1_stop, C1_inc);
fprintf(output,"R1 %le %le %le \n", R1_start, R1_stop, R1_inc);
fprintf(output,"L1 %le %le %le \n", L1_start, L1_stop, L1_inc1);
fprintf(output,"L2 %le %le %le \n", L2_start, L2_stop, L2_inc);
fprintf(output,"K12 %le %le %le \n", K12_start, K12_stop, K12_inc);
fprintf(output,"K23 %le %le %le \n", K23_start, K23_stop, K23_inc);
fprintf(output,"K13 %le %le %le \n", K13_start, K13_stop, K13_inc);
fprintf(output,"R2 %le %le %le \n", R2_start, R2_stop, R2_inc);
fprintf(output,"C2 %le %le %le \n", C2_start, C2_stop, C2_inc);
fprintf(output,"L3 %le %le %le \n", L3_start, L3_stop, L3_inc);
fprintf(output,"R3 %le %le %le \n", R3_start, R3_stop, R3_inc);
fprintf(output,"C3 %le %le %le \n", C3_start, C3_stop, C3_inc);
fprintf(output,"C4 %le %le %le \n", C4_start, C4_stop, C4_inc);
fprintf(output,"R4 %le %le %le \n", R4_start, R4_stop, R4_inc);
fprintf(output,"T1 %le %le %le \n", T1_start, T1_stop, T1_inc1);
fprintf(output,"Vrail %le \n", Vrail);
fprintf(output,"VCpri_init %le \n", VCpri_init);
fprintf(output,"BPS %le \n", BPS);
fprintf(output,"DwellTime %le %le %le \n", DwellTime_start, DwellTime_stop, DwellTime_inc);
fprintf(output,"Current_Limit %le \n", Current_Limit);
fprintf(output,"CoilPower_Limit %le \n", CoilPower_Limit);
fprintf(output,"Goal Type %i \n", Goal_Type);

fclose(output);

data=fopen(filename4,"w");
fprintf(data,"Model Number,C1,R1,L1,L2,K12,K23,K13,R2,C2,L3,R3,C3,C4,R4,BPS,Dwell Time,abs(ILp_max),ICp_rms,abs(VCp_max),abs(VCs_max),abs(VCt_max),Coil Power,Load Power,Fo_pri,Fo_sec,LERT\n");
fclose(data);



LoopTotal = 1;
LoopTotal = LoopTotal * (abs((C1_stop - C1_start) / C1_inc) + 1);
LoopTotal = LoopTotal * (abs((R1_stop - R1_start) / R1_inc) + 1);
LoopTotal = LoopTotal * (abs((L2_stop - L2_start) / L2_inc) + 1);
LoopTotal = LoopTotal * (abs((K12_stop - K12_start) / K12_inc) + 1);
LoopTotal = LoopTotal * (abs((K23_stop - K23_start) / K23_inc) + 1);
LoopTotal = LoopTotal * (abs((K13_stop - K13_start) / K13_inc) + 1);
LoopTotal = LoopTotal * (abs((R2_stop - R2_start) / R2_inc) + 1);
LoopTotal = LoopTotal * (abs((C2_stop - C2_start) / C2_inc) + 1);
LoopTotal = LoopTotal * (abs((C3_stop - C3_start) / C3_inc) + 1);
LoopTotal = LoopTotal * (abs((L3_stop - L3_start) / L3_inc) + 1);
LoopTotal = LoopTotal * (abs((R3_stop - R3_start) / R3_inc) + 1);
LoopTotal = LoopTotal * (abs((C3_stop - C3_start) / C3_inc) + 1);
LoopTotal = LoopTotal * (abs((C4_stop - C4_start) / C4_inc) + 1);
LoopTotal = LoopTotal * (abs((R4_stop - R4_start) / R4_inc) + 1);
LoopTotal = LoopTotal * (abs((L1_stop - L1_start) / L1_inc1) + 1);
LoopTotal = LoopTotal * (abs((DwellTime_stop - DwellTime_start) / DwellTime_inc) + 1);



// Main iteration loops that scan the parameters

for (C1 = C1_start; C1 <= C1_stop; C1 = C1 + C1_inc)
		{
for (R1 = R1_start; R1 <= R1_stop; R1 = R1 + R1_inc)
		{
for (L2 = L2_start; L2 <= L2_stop; L2 = L2 + L2_inc)
		{
for (K12 = K12_start; K12 <= K12_stop; K12 = K12 + K12_inc)
		{
for (K23 = K23_start; K23 <= K23_stop; K23 = K23 + K23_inc)
		{
for (K13 = K13_start; K13 <= K13_stop; K13 = K13 + K13_inc)
		{
for (R2 = R2_start; R2 <= R2_stop; R2 = R2 + R2_inc)
		{
for (C2 = C2_start; C2 <= C2_stop; C2 = C2 + C2_inc)
		{
for (L3 = L3_start; L3 <= L3_stop; L3 = L3 + L3_inc)
		{
for (R3 = R3_start; R3 <= R3_stop; R3 = R3 + R3_inc)
		{
for (C3 = C3_start; C3 <= C3_stop; C3 = C3 + C3_inc)
		{
for (C4 = C4_start; C4 <= C4_stop; C4 = C4 + C4_inc)
		{
for (R4 = R4_start; R4 <= R4_stop; R4 = R4 + R4_inc)
		{
L1_inc = L1_inc1;
for (L1 = L1_start; L1 <= L1_stop; L1 = L1 + L1_inc)
		{
for (DwellTime = DwellTime_start; DwellTime <= DwellTime_stop; DwellTime = DwellTime + DwellTime_inc)
		{

if (T1_inc1 < 0) T1_inc = 2*pi*sqrt(L1*C1)/100; else T1_inc = T1_inc1;  // Automatic T1_inc

// Main calculation engine that finds the output paramters.

//Determin the model with matrix calculations

/* Inverts [L] matrix */
  G[1][1]=L1;
  G[2][2]=L2;
  G[3][3]=L3;
  G[2][1]=K12*sqrt(L1*L2);
  G[3][1]=K13*sqrt(L1*L3);
  G[3][2]=K23*sqrt(L2*L3);
  G[1][2]=G[2][1];
  G[1][3]=G[3][1];
  G[2][3]=G[3][2];
  Invert(3,G);
  /* Assembles the state equations dx/dt=[A]*x+B*vin */
  A[1][1]=0; A[1][2]=0; A[1][3]=0; A[1][4]=0; A[1][5]=1/C1; A[1][6]=0; A[1][7]=0;
  A[2][1]=0; A[2][2]=0; A[2][3]=0; A[2][4]=0; A[2][5]=0; A[2][6]=-1/C2; A[2][7]=1/C2;
  A[3][1]=0; A[3][2]=0; A[3][3]=-1/(R4*C3); A[3][4]=1/(R4*C3); A[3][5]=0; A[3][6]=0; A[3][7]=-1/C3;
  A[4][1]=0; A[4][2]=0; A[4][3]=1/(R4*C4); A[4][4]=-1/(R4*C4); A[4][5]=0; A[4][6]=0; A[4][7]=0;
  A[5][1]=-G[1][1]; A[5][2]=G[1][2]-G[1][3]; A[5][3]=G[1][3]; A[5][4]=0; A[5][5]=-G[1][1]*R1; A[5][6]=-G[1][2]*R2; A[5][7]=-G[1][3]*R3;
  A[6][1]=-G[2][1]; A[6][2]=G[2][2]-G[2][3]; A[6][3]=G[2][3]; A[6][4]=0; A[6][5]=-G[2][1]*R1; A[6][6]=-G[2][2]*R2; A[6][7]=-G[2][3]*R3;
  A[7][1]=-G[3][1]; A[7][2]=G[3][2]-G[3][3]; A[7][3]=G[3][3]; A[7][4]=0; A[7][5]=-G[3][1]*R1; A[7][6]=-G[3][2]*R2; A[7][7]=-G[3][3]*R3;
  B[1]=0; B[2]=0; B[3]=0; B[4]=0; B[5]=G[1][1]; B[6]=G[2][1]; B[7]=G[3][1];
  /*
   /* Generates MT=([I]-(dt/2)*[A])^(-1) */
  for (i=1; i<8; i++)
    for (j=1; j<8; j++)
      if (i==j) MT[i][j]=1-(T1_inc/2)*A[i][j];
      else MT[i][j]=-(T1_inc/2)*A[i][j];
  Invert(7,MT);
  /* Sets the initial state */
  for (i=1; i<7; i++) X0[i]=0;


printf("%ld/%ld ", ModelNumber, LoopTotal);
if (fail == 0) printf("Goal   \r");
if (fail == 1) printf("Current\r");
if (fail == 2) printf("Power  \r");


// Reset varibles/run


//SPECIAL!!  Limit and adjust coil to 50kW
//VCpri_init = sqrt(50000 * 2 / C1 / BPS);


  	goal = 0.0;
  	fail = 0;
  	VCpri_max=0.0;
  	ILpri_max=0.0;
  	VCsec_max = 0.0;
	VCtri_max = 0.0;
  	ModelNumber++;
  	T1_high = 0.0;
  	CoilPower = CoilPower = 0.5 * C1 * VCpri_init * VCpri_init * BPS; /* Default to conventional coil case */
	Cpri_RMSi = 0.0;
  	Eload = 0.0;
	Vin_old = 0.0;
	Vin = 0.0;
  	for (j=1; j<8; j++) {X0[j]=0.0; X1[j]=0.0; Xn[j]=0.0;}

  	X0[1] = VCpri_init;  //this sets initial conditions on Cprimary


// Actual model run over time T1_start to T1_stop.

	for (T1 = T1_start; T1 <= T1_stop; T1 = T1 + T1_inc)
{
  Vin_old = Vin;
  if (X0[5] >= 0) Vin=Vrail; else Vin=-Vrail;	//Inputs Vin for DRSSTC case
  if (T1 > DwellTime) Vin = -Vin;					//Dwell time shutdown
  if ((T1 > DwellTime) && ((Vin > 0) && (X0[5] > 0))) X0[5] = 0.0;	//Antiparallel diodes in DRSSTC
  if ((T1 > DwellTime) && ((Vin < 0) && (X0[5] < 0))) X0[5] = 0.0;	//Antiparallel diodes in DRSSTC

      /* X1=X0+(dt/2)*[A]*X0+(dt/2)*B*(vin(t0)+vin(t0+dt)) */
      for (j=1; j<8; j++) {
        X1[j]=X0[j];
        for (k=1; k<8; k++) X1[j]+=(T1_inc/2)*(A[j][k]*X0[k]);
        X1[j]+=(T1_inc/2)*B[j]*(Vin_old+Vin);
      }
      /* Xn=[MT]*X1 */
      for (j=1; j<8; j++) {
        Xn[j]=0;
        for (k=1; k<8; k++) Xn[j]+=MT[j][k]*X1[k];
      }
      for (j=1; j<8; j++) X0[j]=Xn[j];




/* Output paramters are now calculated and availiable per this time step

X0[1] = VC1
X0[2] = VC2
X0[3] = VC3
X0[4] = VC4
X0[5] = IL1
X0[6] = IL2
Xo[7] = IL3

*/

// Do the checks and calculations per this time step.


	  	if (fabs(X0[5]) > Current_Limit) fail = 1; /*Fail the model if the primary current trips */

	  	if (VCpri_init == 0.0) CoilPower += Vin*X0[5]*T1_inc; /* DRSSTC case */

	  	Eload += (X0[3]-X0[4])*(X0[3]-X0[4])/R4*T1_inc;    /*Load energy/bang */
		Cpri_RMSi += X0[5] * X0[5] * T1_inc;


	  	if (fabs(X0[5]) > fabs(ILpri_max)) ILpri_max = X0[5];		/*max value traps */
      	if (fabs(X0[1]) > fabs(VCpri_max)) VCpri_max = X0[1];
	  	if (fabs(X0[2]) > fabs(VCsec_max)) VCsec_max = X0[2];
		if (fabs(X0[3]) > fabs(VCtri_max)) VCtri_max = X0[3];

		if (Goal_Type == 0) {if (fabs(X0[3]) > fabs(goal)) {goal = X0[3]; T1_high = T1;}}	/*Goal Seek function (Vsec_max)*/


}  //T1 loop end

	Cpri_RMSi = sqrt(Cpri_RMSi)*sqrt(BPS);  // Finish RMS current calculation
	if (VCpri_init == 0.0) CoilPower = CoilPower * BPS;

		if (Goal_Type == 1) {if (fabs(Eload) > fabs(goal)) {goal = Eload; T1_high = T1;}}	/*Goal Seek function (Eload)*/

		if (CoilPower > CoilPower_Limit) fail = 2;  /*fail the model if the bang energy is too high */





//  Check if we have a new winning goal, report it if yes.

  if ((fabs(goal) > fabs(goal_max) && (fail == 0)) || (Goal_Type == 2))
		{


			// Screen output


		if (Goal_Type != 2 )  //Goal_Type 2 output supression
		{

			goal_max = goal;
			GoalNum = GoalNum + 1;
			printf("\n\n");  // <- Use if conio.h is not avaialable instead of next two lines.
			//if (screen == 0) {screen = 1; clrscr();} // Clear screen on first display
			//gotoxy(1,1); //puts cursor at top left of screen
			printf(REV "\n");
			printf("Goal = %18.9lf  ", goal_max);
			if (Goal_Type == 0) printf("Maxium Secondary Voltage \n");
			if (Goal_Type == 1) printf("Maxium Load Energy \n");
			printf("Goal Time = %le \n", T1_high);
			printf("Model Number = %ld \n", ModelNumber);
			printf("Goal Number = %ld \n", GoalNum);
			printf("Cprimary = %le \n", C1);
			printf("Lprimary = %le \n", L1);
			printf("Rprimary = %le \n", R1);
			printf("K12 = %lf   K23 = %lf   K13 = %lf \n" , K12, K23, K13);
			printf("Csecondary = %le \n", C2);
			printf("Lsecondary = %le \n", L2);
			printf("Rsecondary = %le \n", R2);
			printf("Ltertiary = %le \n", L3);
			printf("Ctertiary = %le \n", C3);
			printf("Rtertiary = %le \n", R3);
			printf("Cload = %le \n" , C4);
			printf("Rload = %le \n", R4);
			printf("BPS = %lf \n", BPS);
			printf("Dwell Time = %le \n", DwellTime);
			printf("Ilprimay Maximum = %lf \n", ILpri_max);
			printf("ICprimary RMS = %lf \n", Cpri_RMSi);
			printf("VCprimary Maximum = %lf \n", VCpri_max);
			printf("VCsecondary Maximum = %lf \n", VCsec_max);
			printf("VCtertiary Maximum = %1f \n", VCtri_max);
			printf("Coil Power = %lf   Primary Bang Energy = %lf \n", CoilPower, CoilPower / BPS);
			printf("Load Power = %lf   Load Bang Energy = %lf \n", Eload*BPS, Eload);
			printf("Primary F0 = %lf\n",1/(2 * pi * sqrt(L1 * C1)));


			//File output

			output=fopen(filename1,"a");
			fprintf(output,"\n\n\n");
			fprintf(output, REV "\n");
			fprintf(output,"Goal = %18.9lf  ", goal_max);
			if (Goal_Type == 0) fprintf(output,"Maxium Secondary Voltage \n");
			if (Goal_Type == 1) fprintf(output,"Maxium Load Energy \n");
			fprintf(output,"Goal Time = %le \n", T1_high);
			fprintf(output,"Model Number = %ld \n", ModelNumber);
			fprintf(output,"Goal Number = %ld \n\n", GoalNum);
			fprintf(output,"Cprimary = %le \n", C1);
			fprintf(output,"Lprimary = %le \n", L1);
			fprintf(output,"Rprimary = %le \n", R1);
			fprintf(output,"K12 = %lf   K23 = %lf   K13 = %lf \n" , K12, K23, K13);
			fprintf(output,"Csecondary = %le \n", C2);
			fprintf(output,"Lsecondary = %le \n", L2);
			fprintf(output,"Rsecondary = %le \n", R2);
			fprintf(output,"Ltertiary = %le \n", L3);
			fprintf(output,"Ctertiary = %le \n", C3);
			fprintf(output,"Rtertiary = %le \n", R3);
			fprintf(output,"Cload = %le \n" , C4);
			fprintf(output,"Rload = %le \n", R4);
			fprintf(output,"BPS = %lf \n", BPS);
			fprintf(output,"Dwell Time = %le \n\n", DwellTime);
			fprintf(output,"Ilprimay Maximum = %lf \n", ILpri_max);
			fprintf(output,"ICprimary RMS = %lf \n", Cpri_RMSi);
			fprintf(output,"VCprimary Maximum = %lf \n", VCpri_max);
			fprintf(output,"VCsecondary Maximum = %lf \n", VCsec_max);
			fprintf(output,"VCtertiary Maximum = %1f \n", VCtri_max);
			fprintf(output,"Coil Power = %lf   Primary Bang Energy = %lf \n", CoilPower, CoilPower / BPS);
			fprintf(output,"Load Power = %lf   Load Bang Energy = %lf \n", Eload*BPS, Eload);
			fprintf(output,"Primary F0 = %lf\n",1/(2 * pi * sqrt(L1 * C1)));
			fclose(output);




			// Do the last model over again to record the values to a file and check LERT.

			option=fopen(filename3,"w");
			fprintf(output,"Time , Vin , VCpri , VCsec , VCtri , VCload , ILpri , ILsec , ILtri, Eload (J/bang) \n");
			fclose(option);

			option=fopen(filename3,"a");

		}  // goal_type = 2 output supression return

			// Sets the initial states
  				for (j=1; j<8; j++) {X0[j]=0.0; X1[j]=0.0; Xn[j]=0.0;}

  				X0[1] = VCpri_init;  //this sets initial conditions on Cprimary

				Eload0 = Eload * 0.1;  //Set variables for Load Energy Rise Time
				Eload1 = Eload * 0.9;	// 10% to 90% rise time values
				TEload = 0.0;
				TEload0 = 0.0;
				TEload1 = 0.0;
				Eload = 0.0;
				Vin = 0.0;
				Vin_old = 0.0;

				// Rerun the model in special T1 loop

				for (T1 = T1_start; T1 <= T1_stop; T1 = T1 + T1_inc)
  				{

					Vin_old = Vin;
  					if (X0[5] >= 0) Vin=Vrail; else Vin=-Vrail;	//Inputs Vin for DRSSTC case
  					if (T1 > DwellTime) Vin = -Vin;				//Dwell time shutdown
					if ((T1 > DwellTime) && ((Vin > 0) && (X0[5] > 0))) X0[5] = 0.0;	//Antiparallel diodes in DRSSTC
  					if ((T1 > DwellTime) && ((Vin < 0) && (X0[5] < 0))) X0[5] = 0.0;	//Antiparallel diodes in DRSSTC

      				// X1=X0+(dt/2)*[A]*X0+(dt/2)*B*(vin(t0)+vin(t0+dt))
      				for (j=1; j<8; j++) {
        			X1[j]=X0[j];
        			for (k=1; k<8; k++) X1[j]+=(T1_inc/2)*(A[j][k]*X0[k]);
        			X1[j]+=(T1_inc/2)*B[j]*(Vin_old+Vin);
      				}
      				// Xn=[MT]*X1
      				for (j=1; j<8; j++) {
        			Xn[j]=0;
        			for (k=1; k<8; k++) Xn[j]+=MT[j][k]*X1[k];
      				}
      				for (j=1; j<8; j++) X0[j]=Xn[j];

				//  End of T1 step calculation.  X0[] varaibles now available.

			if (Goal_Type != 2)
			{

					// Print X0[] values to a file
					// Only print out every 4 iteretions
					if ((PrintNumber % 4) == 0) fprintf(output,"%le , %le , %le , %le , %le , %le , %le , %le , %le, %le \n", T1,Vin,X0[1],X0[2],X0[3],X0[4],X0[5],X0[6],X0[7],Eload);
					PrintNumber++;
			}	// goal_type = 2 output supression return
					// Find Eload and LERT numbers
					Eload += (X0[3]-X0[4])*(X0[3]-X0[4])/R4*T1_inc;    //Load power per bang
					if ((Eload > Eload0) && (TEload0 == 0.0)) TEload0 = T1;
					if ((Eload > Eload1) && (TEload1 == 0.0)) {TEload1 = T1; TEload = TEload1 - TEload0;}

				} 	// End of special T1 loop


			if (Goal_Type != 2)  //Goal_Type = 2 output supression
			{

				fclose(option);

				printf("Load Energy Rise Time (Sec) = %le \n", TEload);  // Report LERT

				output=fopen(filename1,"a");
				fprintf(output,"Load Energy Rise Time (Sec) = %le \n", TEload);
				fclose(output);
			} // goal_type = 2 output supression return

				data=fopen(filename4,"a");
				fprintf(data,"%li,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le,%le\n", ModelNumber,C1,R1,L1,L2,K12,K23,K13,R2,C2,L3,R3,C3,
				C4,R4,BPS,DwellTime,fabs(ILpri_max),Cpri_RMSi,fabs(VCpri_max),fabs(VCsec_max),fabs(VCtri_max),CoilPower,Eload*BPS,1/(2 * pi * sqrt(L1 * C1)),1/(2 * pi * sqrt(L2 * C2)),TEload);
				fclose(data);

		}  //End of goal reporting branch




}}}}}}}}}}}}}}} // Returns for C1 through DwellTime loops


	printf("Models Tested = %ld \n", ModelNumber);



	output=fopen(filename1,"a");
	fprintf(output,"Models Tested = %ld \n", ModelNumber);
	fclose(output);


    return(1);



}  // End of Main

